@@ -1,8 +1,9 @@
-/*************************************************************************
+/*******************************************************************************
 
-"FreePastry" Peer-to-Peer Application Development Substrate 
+"FreePastry" Peer-to-Peer Application Development Substrate
 
-Copyright 2002, Rice University. All rights reserved.
+Copyright 2002-2007, Rice University. Copyright 2006-2007, Max Planck Institute 
+for Software Systems.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -15,30 +16,30 @@ notice, this list of conditions and the following disclaimer.
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 
-- Neither  the name  of Rice  University (RICE) nor  the names  of its
-contributors may be  used to endorse or promote  products derived from
-this software without specific prior written permission.
+- Neither the name of Rice  University (RICE), Max Planck Institute for Software 
+Systems (MPI-SWS) nor the names of its contributors may be used to endorse or 
+promote products derived from this software without specific prior written 
+permission.
 
-This software is provided by RICE and the contributors on an "as is"
-basis, without any representations or warranties of any kind, express
-or implied including, but not limited to, representations or
-warranties of non-infringement, merchantability or fitness for a
-particular purpose. In no event shall RICE or contributors be liable
-for any direct, indirect, incidental, special, exemplary, or
-consequential damages (including, but not limited to, procurement of
-substitute goods or services; loss of use, data, or profits; or
-business interruption) however caused and on any theory of liability,
-whether in contract, strict liability, or tort (including negligence
-or otherwise) arising in any way out of the use of this software, even
-if advised of the possibility of such damage.
+This software is provided by RICE, MPI-SWS and the contributors on an "as is" 
+basis, without any representations or warranties of any kind, express or implied 
+including, but not limited to, representations or warranties of 
+non-infringement, merchantability or fitness for a particular purpose. In no 
+event shall RICE, MPI-SWS or contributors be liable for any direct, indirect, 
+incidental, special, exemplary, or consequential damages (including, but not 
+limited to, procurement of substitute goods or services; loss of use, data, or 
+profits; or business interruption) however caused and on any theory of 
+liability, whether in contract, strict liability, or tort (including negligence
+or otherwise) arising in any way out of the use of this software, even if 
+advised of the possibility of such damage.
 
-********************************************************************************/
+*******************************************************************************/ 
 package rice.pastry.standard;
 
 import rice.environment.logging.Logger;
 import rice.pastry.*;
 import rice.pastry.messaging.*;
-import rice.pastry.security.*;
+import rice.pastry.client.PastryAppl;
 import rice.pastry.leafset.*;
 import rice.pastry.routing.*;
 
@@ -46,96 +47,44 @@ import java.util.*;
 
 /**
  * An implementation of a simple leaf set protocol.
- *
- * @version $Id: StandardLeafSetProtocol.java 3038 2006-02-07 10:01:01Z jeffh $
+ * 
+ * @version $Id: StandardLeafSetProtocol.java 3613 2007-02-15 14:45:14Z jstewart $
+ * 
  * @author Peter Druschel
  * @author Andrew Ladd
  */
 
-public class StandardLeafSetProtocol implements MessageReceiver {
-  /**
-   * DESCRIBE THE FIELD
-   */
+public class StandardLeafSetProtocol extends PastryAppl implements Observer {
   protected final boolean failstop = true;
 
   // nodes are assumed to fail silently
 
-  /**
-   * DESCRIBE THE FIELD
-   */
-  protected NodeHandle localHandle;
-
-  /**
-   * DESCRIBE THE FIELD
-   */
-  protected PastryNode localNode;
-
-  /**
-   * DESCRIBE THE FIELD
-   */
-  protected PastrySecurityManager security;
-
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected LeafSet leafSet;
 
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected RoutingTable routeTable;
 
-  private Address address;
-
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected Logger logger;
 
-  /**
-   * Optimization. gc was thrashing due to all of these HashSets being created.
-   * So, now we just reuse one.
-   */
-  HashSet cachedSet = null;
-
-  /**
-   * Constructor for StandardLeafSetProtocol.
-   *
-   * @param ln DESCRIBE THE PARAMETER
-   * @param local DESCRIBE THE PARAMETER
-   * @param sm DESCRIBE THE PARAMETER
-   * @param ls DESCRIBE THE PARAMETER
-   * @param rt DESCRIBE THE PARAMETER
-   */
   public StandardLeafSetProtocol(PastryNode ln, NodeHandle local,
-                                 PastrySecurityManager sm, LeafSet ls, RoutingTable rt) {
-    localNode = ln;
-    localHandle = local;
-    security = sm;
+      LeafSet ls, RoutingTable rt) {
+    super(ln, LeafSetProtocolAddress.getCode());
     leafSet = ls;
-    cachedSet = new HashSet(leafSet.maxSize() * 2);
+    Iterator i = leafSet.asList().iterator();
+    while(i.hasNext()) {
+      NodeHandle nh = (NodeHandle)i.next(); 
+      nh.addObserver(this);
+    }    
+    cachedSet = new HashSet(leafSet.maxSize() * 2);  
     routeTable = rt;
     logger = ln.getEnvironment().getLogManager().getLogger(getClass(), null);
-    address = new LeafSetProtocolAddress();
-  }
-
-  /**
-   * Gets the address.
-   *
-   * @return the address.
-   */
-
-  public Address getAddress() {
-    return address;
   }
 
   /**
    * Receives messages.
-   *
+   * 
    * @param msg the message.
    */
-
-  public void receiveMessage(Message msg) {
+  public void messageForAppl(Message msg) {
     if (msg instanceof BroadcastLeafSet) {
       // receive a leafset from another node
       BroadcastLeafSet bls = (BroadcastLeafSet) msg;
@@ -155,9 +104,8 @@ public class StandardLeafSetProtocol implements MessageReceiver {
       if (!failstop) {
         // with arbitrary node failures, we need to broadcast whenever we learn
         // something new
-        if (changed) {
+        if (changed)
           broadcast();
-        }
 
         // then, send ls to sending node if that node's ls is missing nodes
         checkLeafSet(remotels, from, false);
@@ -165,15 +113,13 @@ public class StandardLeafSetProtocol implements MessageReceiver {
       }
 
       // if this node has just joined, notify the members of our new leaf set
-      if (type == BroadcastLeafSet.JoinInitial) {
+      if (type == BroadcastLeafSet.JoinInitial)
         broadcast();
-      }
 
       // if we receive a correction to a leafset we have sent out, notify the
       // members of our new leaf set
-      if (type == BroadcastLeafSet.Correction && changed) {
+      if (type == BroadcastLeafSet.Correction && changed)
         broadcast();
-      }
 
       // Check if any of our local leaf set members are missing in the received
       // leaf set
@@ -188,13 +134,12 @@ public class StandardLeafSetProtocol implements MessageReceiver {
       RequestLeafSet rls = (RequestLeafSet) msg;
 
       NodeHandle returnHandle = rls.returnHandle();
-      returnHandle = security.verifyNodeHandle(returnHandle);
 
       if (returnHandle.isAlive()) {
-        BroadcastLeafSet bls = new BroadcastLeafSet(localHandle, leafSet,
-          BroadcastLeafSet.Update);
+        BroadcastLeafSet bls = new BroadcastLeafSet(thePastryNode.getLocalHandle(), leafSet,
+            BroadcastLeafSet.Update, rls.getTimeStamp());
 
-        returnHandle.receiveMessage(bls);
+        thePastryNode.send(returnHandle,bls);
       }
     } else if (msg instanceof InitiateLeafSetMaintenance) {
       // request for leafset maintenance
@@ -202,21 +147,27 @@ public class StandardLeafSetProtocol implements MessageReceiver {
       // perform leafset maintenance
       maintainLeafSet();
 
-    } else {
+    } else
       throw new Error("message received is of unknown type");
-    }
   }
 
+
+  /**
+   * Optimization. 
+   * gc was thrashing due to all of these HashSets being created.  So, now we just reuse one.
+   */
+  HashSet cachedSet = null;
+  
   /**
    * Checks a received leafset advertisement for missing nodes
-   *
+   * 
    * @param remotels the remote leafset
    * @param from the node from which we received the leafset
    * @param notifyMissing if true, notify missing nodes
    * @return true if any nodes where found missing in the received leafset
    */
   protected boolean checkLeafSet(LeafSet remotels, NodeHandle from,
-                                 boolean notifyMissing) {
+      boolean notifyMissing) {
 
     // check if any of our local leaf set members are missing in the received
     // leaf set
@@ -229,20 +180,18 @@ public class StandardLeafSetProtocol implements MessageReceiver {
     HashSet insertedHandles;
     if (notifyMissing) {
       cachedSet.clear();
-      insertedHandles = cachedSet;
-    } else {
+      insertedHandles = cachedSet;      
+    } else
       insertedHandles = null;
-    }
 
-    BroadcastLeafSet bl = new BroadcastLeafSet(localHandle, leafSet,
-      BroadcastLeafSet.Correction);
-    boolean changed = remotels.merge(leafSet, localHandle, null, security,
-      true, insertedHandles);
+    BroadcastLeafSet bl = new BroadcastLeafSet(thePastryNode.getLocalHandle(), leafSet,
+        BroadcastLeafSet.Correction, 0);
+    boolean changed = remotels.merge(leafSet, thePastryNode.getLocalHandle(), null,
+        true, insertedHandles);
 
     if (changed) {
       // nodes where missing, send update to "from"
-      from = security.verifyNodeHandle(from);
-      from.receiveMessage(bl);
+      thePastryNode.send(from,bl);
 
       if (notifyMissing) {
         // send leafset to nodes that where missing from remotels
@@ -254,7 +203,7 @@ public class StandardLeafSetProtocol implements MessageReceiver {
         while (it.hasNext()) {
           // send leafset to missing node
           NodeHandle nh = (NodeHandle) it.next();
-          nh.receiveMessage(bl);
+          thePastryNode.send(nh,bl);
         }
       }
     }
@@ -264,14 +213,14 @@ public class StandardLeafSetProtocol implements MessageReceiver {
 
   /**
    * Merge a remote leafset into our own
-   *
+   * 
    * @param remotels the remote leafset
    * @param from the node from which we received the leafset
    * @return true if the leafset changed
    */
 
   protected boolean mergeLeafSet(LeafSet remotels, NodeHandle from) {
-    return leafSet.merge(remotels, from, routeTable, security, false, null);
+    return leafSet.merge(remotels, from, routeTable, false, null);
   }
 
   /**
@@ -284,29 +233,27 @@ public class StandardLeafSetProtocol implements MessageReceiver {
 
   /**
    * Broadcast the leaf set to all members of the local leaf set.
-   *
+   * 
    * @param type the type of broadcast message used
    */
 
   protected void broadcast(int type) {
-    BroadcastLeafSet bls = new BroadcastLeafSet(localHandle, leafSet, type);
+    BroadcastLeafSet bls = new BroadcastLeafSet(thePastryNode.getLocalHandle(), leafSet, type, 0);
 
     int cwSize = leafSet.cwSize();
     int ccwSize = leafSet.ccwSize();
     IdSet sent = new IdSet();
 
     for (int i = -ccwSize; i <= cwSize; i++) {
-      if (i == 0) {
+      if (i == 0)
         continue;
-      }
 
       NodeHandle nh = leafSet.get(i);
-      if (nh == null || nh.isAlive() == false) {
+      if (nh == null || nh.isAlive() == false)
         continue;
-      }
 
       if (!sent.isMember(nh.getNodeId())) {
-        nh.receiveMessage(bls);
+        thePastryNode.send(nh,bls);
         sent.addMember(nh.getNodeId());
       }
     }
@@ -315,14 +262,14 @@ public class StandardLeafSetProtocol implements MessageReceiver {
   /**
    * Broadcast the local leaf set to all members of the given leaf set, plus the
    * node from which the leaf set was received.
-   *
+   * 
    * @param ls the leafset whose members we send to local leaf set
    * @param from the node from which ls was received
    */
 
   protected void broadcast(LeafSet ls, NodeHandle from) {
-    BroadcastLeafSet bls = new BroadcastLeafSet(localHandle, leafSet,
-      BroadcastLeafSet.JoinAdvertise);
+    BroadcastLeafSet bls = new BroadcastLeafSet(thePastryNode.getLocalHandle(), leafSet,
+        BroadcastLeafSet.JoinAdvertise, 0);
 
     int cwSize = ls.cwSize();
     int ccwSize = ls.ccwSize();
@@ -330,19 +277,15 @@ public class StandardLeafSetProtocol implements MessageReceiver {
     for (int i = -ccwSize; i <= cwSize; i++) {
       NodeHandle nh;
 
-      if (i == 0) {
+      if (i == 0)
         nh = from;
-      } else {
+      else
         nh = ls.get(i);
-      }
 
-      if (nh == null || nh.isAlive() == false) {
+      if (nh == null || nh.isAlive() == false)
         continue;
-      }
 
-      nh = security.verifyNodeHandle(nh);
-
-      nh.receiveMessage(bls);
+      thePastryNode.send(nh,bls);
 
     }
   }
@@ -353,10 +296,8 @@ public class StandardLeafSetProtocol implements MessageReceiver {
    * periodically.
    */
   public void maintainLeafSet() {
-    if (logger.level <= Logger.FINE) {
-      logger.log(
-        "maintainLeafSet " + localHandle.getNodeId());
-    }
+    if (logger.level <= Logger.FINE) logger.log(
+        "maintainLeafSet " + thePastryNode.getLocalHandle().getNodeId());
 
     boolean lostMembers = false;
 
@@ -384,10 +325,9 @@ public class StandardLeafSetProtocol implements MessageReceiver {
     // if we lost entries, or the size is below max and we don't span the entire
     // ring then
     // request the leafset from other leafset members
-    if (lostMembers ||
-    // (leafSet.size() < leafSet.maxSize() &&
-    // !leafSet.overlaps())) {
-    (leafSet.size() < leafSet.maxSize())) {
+    if (lostMembers || // (leafSet.size() < leafSet.maxSize() &&
+                        // !leafSet.overlaps())) {
+        (leafSet.size() < leafSet.maxSize())) {
 
       // request leaf sets
       requestLeafSet();
@@ -400,7 +340,7 @@ public class StandardLeafSetProtocol implements MessageReceiver {
 
   private void requestLeafSet() {
 
-    RequestLeafSet rls = new RequestLeafSet(localHandle);
+    RequestLeafSet rls = new RequestLeafSet(thePastryNode.getLocalHandle(), thePastryNode.getEnvironment().getTimeSource().currentTimeMillis());
     int cwSize = leafSet.cwSize();
     int ccwSize = leafSet.ccwSize();
     boolean allDead = true;
@@ -409,38 +349,63 @@ public class StandardLeafSetProtocol implements MessageReceiver {
     for (int i = -ccwSize; i < 0; i++) {
       NodeHandle handle = leafSet.get(i);
       if (handle != null && handle.isAlive()) {
-        handle.receiveMessage(rls);
+        thePastryNode.send(handle,rls);
         allDead = false;
         break;
       }
     }
 
-    if (allDead && leafSet.size() > 0) {
-      if (logger.level <= Logger.SEVERE) {
-        logger.log(
-          "Ring failure at" + localHandle.getNodeId()
-          + "all ccw leafset entries failed");
-      }
-    }
+    if (allDead && leafSet.size() > 0)
+      if (logger.level <= Logger.SEVERE) logger.log(
+          "Ring failure at" + thePastryNode.getLocalHandle().getNodeId()
+              + "all ccw leafset entries failed");
 
     allDead = true;
     // request from most distant live cw entry
     for (int i = cwSize; i > 0; i--) {
       NodeHandle handle = leafSet.get(i);
       if (handle != null && handle.isAlive()) {
-        handle.receiveMessage(rls);
+        thePastryNode.send(handle,rls);
         allDead = false;
         break;
       }
     }
 
-    if (allDead && leafSet.size() > 0) {
-      if (logger.level <= Logger.SEVERE) {
-        logger.log(
-          "Ring failure at" + localHandle.getNodeId()
-          + "all cw leafset entries failed");
-      }
+    if (allDead && leafSet.size() > 0)
+      if (logger.level <= Logger.SEVERE) logger.log(
+          "Ring failure at" + thePastryNode.getLocalHandle().getNodeId()
+              + "all cw leafset entries failed");
+
+  }
+
+  public boolean deliverWhenNotReady() {
+    return true;
+  }
+
+  /**
+   * We manage the leafset now.  Implementation is simple, remove items that die.
+   * 
+   * We should be observers on everything in the leafset, and nothing else.
+   */
+  @Override
+  public void leafSetChange(NodeHandle nh, boolean wasAdded) {
+    super.leafSetChange(nh, wasAdded);
+    if (wasAdded) {
+      nh.addObserver(this); 
+    } else {
+      nh.deleteObserver(this); 
     }
   }
 
+  /**
+   * If nodehandle is dead, remove it from the leafset.
+   */
+  public void update(Observable o, Object arg) {
+//    if (o instanceof NodeHandle) {      
+      if (arg == NodeHandle.DECLARED_DEAD) {
+        leafSet.remove((NodeHandle)o);
+      }
+//    }    
+  }
+
 }
