@@ -1,8 +1,9 @@
-/*************************************************************************
+/*******************************************************************************
 
-"FreePastry" Peer-to-Peer Application Development Substrate 
+"FreePastry" Peer-to-Peer Application Development Substrate
 
-Copyright 2002, Rice University. All rights reserved.
+Copyright 2002-2007, Rice University. Copyright 2006-2007, Max Planck Institute 
+for Software Systems.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -15,24 +16,24 @@ notice, this list of conditions and the following disclaimer.
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 
-- Neither  the name  of Rice  University (RICE) nor  the names  of its
-contributors may be  used to endorse or promote  products derived from
-this software without specific prior written permission.
+- Neither the name of Rice  University (RICE), Max Planck Institute for Software 
+Systems (MPI-SWS) nor the names of its contributors may be used to endorse or 
+promote products derived from this software without specific prior written 
+permission.
 
-This software is provided by RICE and the contributors on an "as is"
-basis, without any representations or warranties of any kind, express
-or implied including, but not limited to, representations or
-warranties of non-infringement, merchantability or fitness for a
-particular purpose. In no event shall RICE or contributors be liable
-for any direct, indirect, incidental, special, exemplary, or
-consequential damages (including, but not limited to, procurement of
-substitute goods or services; loss of use, data, or profits; or
-business interruption) however caused and on any theory of liability,
-whether in contract, strict liability, or tort (including negligence
-or otherwise) arising in any way out of the use of this software, even
-if advised of the possibility of such damage.
+This software is provided by RICE, MPI-SWS and the contributors on an "as is" 
+basis, without any representations or warranties of any kind, express or implied 
+including, but not limited to, representations or warranties of 
+non-infringement, merchantability or fitness for a particular purpose. In no 
+event shall RICE, MPI-SWS or contributors be liable for any direct, indirect, 
+incidental, special, exemplary, or consequential damages (including, but not 
+limited to, procurement of substitute goods or services; loss of use, data, or 
+profits; or business interruption) however caused and on any theory of 
+liability, whether in contract, strict liability, or tort (including negligence
+or otherwise) arising in any way out of the use of this software, even if 
+advised of the possibility of such damage.
 
-********************************************************************************/
+*******************************************************************************/ 
 package rice.pastry.testing;
 
 import java.io.IOException;
@@ -40,6 +41,7 @@ import java.util.*;
 
 import rice.environment.Environment;
 import rice.environment.random.RandomSource;
+import rice.p2p.commonapi.rawserialization.OutputBuffer;
 import rice.p2p.commonapi.RangeCannotBeDeterminedException;
 import rice.pastry.*;
 import rice.pastry.leafset.*;
@@ -48,43 +50,29 @@ import rice.pastry.standard.RandomNodeIdFactory;
 
 /**
  * This class tests the correctness of the leafset in Pastry.
- *
- * @version $Id: LeafSetTest.java 3249 2006-04-27 13:08:32Z jeffh $
+ * 
+ * @version $Id: LeafSetTest.java 3613 2007-02-15 14:45:14Z jstewart $
+ * 
  * @author Alan Mislove
  */
 public class LeafSetTest {
 
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected NodeIdFactory factory;
 
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected RandomSource random;
 
-  /**
-   * Constructor for LeafSetTest.
-   */
   public LeafSetTest() {
     Environment env = new Environment();
     random = env.getRandomSource();
     factory = new RandomNodeIdFactory(env);
   }
 
-  /**
-   * DESCRIBE THE METHOD
-   */
   public void start() {
     testCumulativeRange();
     testNonCumulativeRange();
     testObservers();
   }
 
-  /**
-   * A unit test for JUnit
-   */
   public void testObservers() {
     int halfLeafSet = 3;
     LeafSet leafset = generateLeafSet(halfLeafSet * 2, halfLeafSet * 2, false);
@@ -143,11 +131,47 @@ public class LeafSetTest {
     }
   }
 
+  class MyObserver implements NodeSetListener {
+    LeafSet ls;
+
+    public MyObserver(LeafSet ls) {
+      this.ls = ls;
+    }
+
+    public void nodeSetUpdate(NodeSetEventSource set, NodeHandle handle, boolean added) {
+      SimilarSet caller;
+      if (set instanceof SimilarSet) {
+        caller = (SimilarSet) set;
+      }
+
+      if (ls.overlaps() && (ls.ccwSize() != ls.cwSize())) {
+        System.out.println("FAILURE: overlaps and different size" + ls);
+      }
+      if (added) {
+        boolean consistent = !ls.directTest(handle);
+        if (!consistent) {
+          System.out.println("FAILURE:" + handle
+              + " was added, but ls is inconsistent.");
+          System.out.println(ls);
+        } else {
+          System.out.println("OK:" + handle + " was added");
+        }
+      } else { // node was removed
+        boolean consistent = !ls.member(handle.getNodeId());
+        if (!consistent) {
+          System.out.println("FAILURE:" + handle
+              + " was removed, but ls is inconsistent.");
+          System.out.println(ls);
+        } else {
+          System.out.println("OK:" + handle + " was removed");
+        }
+      }
+    }
+
+  }
+
   /**
    * Throws an exception if the test condition is not met.
-   *
-   * @param intention DESCRIBE THE PARAMETER
-   * @param test DESCRIBE THE PARAMETER
    */
   protected final void assertTrue(String intention, boolean test) {
     if (!test) {
@@ -156,48 +180,30 @@ public class LeafSetTest {
     }
   }
 
-  /**
-   * DESCRIBE THE METHOD
-   *
-   * @param x DESCRIBE THE PARAMETER
-   * @param y DESCRIBE THE PARAMETER
-   * @return DESCRIBE THE RETURN VALUE
-   */
   protected int min(int x, int y) {
-    if (y < x) {
+    if (y < x)
       return y;
-    } else {
+    else
       return x;
-    }
   }
 
-  /**
-   * DESCRIBE THE METHOD
-   *
-   * @param x DESCRIBE THE PARAMETER
-   * @return DESCRIBE THE RETURN VALUE
-   */
   protected int abs(int x) {
-    if (x < 0) {
+    if (x < 0)
       return -x;
-    } else {
+    else
       return x;
-    }
   }
 
-  /**
-   * A unit test for JUnit
-   */
   public void testCumulativeRange() {
     for (int nodes = 2; nodes < 20; nodes++) {
       for (int size = 2; size < 17; size += 2) {
         LeafSet leafset = generateLeafSet(size, nodes, false);
 
         System.out.println("Testing cumulative ranges with " + nodes
-          + " nodes and leafset size of " + size);
+            + " nodes and leafset size of " + size);
 
         for (int pos = -min(size / 2, nodes / 2); pos <= min(size / 2,
-          nodes / 2); pos++) {
+            nodes / 2); pos++) {
 
           for (int q = 0; q < size; q++) {
             IdRange range = null;
@@ -208,23 +214,23 @@ public class LeafSetTest {
             }
             if ((q < size / 2 - abs(pos)) || (size + 1 > nodes)) {
               assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                + nodes + " size " + size + " should be defined in leafset "
-                + leafset, range != null);
+                  + nodes + " size " + size + " should be defined in leafset "
+                  + leafset, range != null);
 
               if (q >= nodes - 1) {
                 assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                  + nodes + " size " + size + " should be full in leafset "
-                  + leafset, range.isFull());
+                    + nodes + " size " + size + " should be full in leafset "
+                    + leafset, range.isFull());
               } else {
                 assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                  + nodes + " size " + size + " should be full in leafset "
-                  + leafset, !range.isFull());
+                    + nodes + " size " + size + " should be full in leafset "
+                    + leafset, !range.isFull());
               }
             } else {
               assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                + nodes + " size " + size
-                + " should be not defined in leafset " + leafset,
-                range == null);
+                  + nodes + " size " + size
+                  + " should be not defined in leafset " + leafset,
+                  range == null);
             }
           }
         }
@@ -232,19 +238,16 @@ public class LeafSetTest {
     }
   }
 
-  /**
-   * A unit test for JUnit
-   */
   public void testNonCumulativeRange() {
     for (int nodes = 1; nodes < 20; nodes++) {
       for (int size = 2; size < 17; size += 2) {
         LeafSet leafset = generateLeafSet(size, nodes, false);
 
         System.out.println("Testing non-cumulative ranges with " + nodes
-          + " nodes and leafset size of " + size);
+            + " nodes and leafset size of " + size);
 
         for (int pos = -min(size / 2, nodes / 2); pos <= min(size / 2,
-          nodes / 2); pos++) {
+            nodes / 2); pos++) {
 
           for (int q = 0; q < size; q++) {
             IdRange range = null;
@@ -254,23 +257,24 @@ public class LeafSetTest {
             }
             if ((q < size / 2 - abs(pos)) || (size + 1 > nodes)) {
               assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                + nodes + " size " + size + " should not be null in leafset "
-                + leafset, range != null);
+                  + nodes + " size " + size + " should not be null in leafset "
+                  + leafset, range != null);
 
               if (q >= nodes) {
                 assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                  + nodes + " size " + size + " should be empty in leafset "
-                  + leafset, range.isEmpty());
+                    + nodes + " size " + size + " should be empty in leafset "
+                    + leafset, range.isEmpty());
               } else {
                 assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                  + nodes + " size " + size
-                  + " should be defined in leafset " + leafset, !range.isEmpty());
+                    + nodes + " size " + size
+                    + " should be defined in leafset " + leafset, !range
+                    .isEmpty());
               }
             } else {
               assertTrue("Range of node " + pos + " with q " + q + " nodes "
-                + nodes + " size " + size
-                + " should be not defined in leafset " + leafset,
-                range == null);
+                  + nodes + " size " + size
+                  + " should be not defined in leafset " + leafset,
+                  range == null);
             }
           }
         }
@@ -284,9 +288,9 @@ public class LeafSetTest {
           }
 
           assertTrue(
-            "Sum of individual ranges should produce entire range with nodes "
-            + nodes + " size " + size + " in leafset " + leafset
-            + " with total " + total, total.isFull());
+              "Sum of individual ranges should produce entire range with nodes "
+                  + nodes + " size " + size + " in leafset " + leafset
+                  + " with total " + total, total.isFull());
         }
       }
     }
@@ -295,11 +299,10 @@ public class LeafSetTest {
   /**
    * Returns a leafset of size size out of a network with the specified number
    * of nodes
-   *
+   * 
    * @param size The size of the leaf set
    * @param nodes The number of nodes in the network
    * @param crossover Whether the leafset must cross over the '0' boundary not
-   * @return DESCRIBE THE RETURN VALUE
    */
   protected LeafSet generateLeafSet(int size, int nodes, boolean crossover) {
     NodeHandle[] handles = new NodeHandle[nodes];
@@ -319,7 +322,7 @@ public class LeafSetTest {
     }
 
     int base = (i + size / 2) % nodes;
-    LeafSet set = new LeafSet(handles[base], size);
+    LeafSet set = new LeafSet(handles[base], size, null);
 
     for (int j = 0; j < nodes; j++) {
       set.put(handles[j]);
@@ -328,134 +331,37 @@ public class LeafSetTest {
     return set;
   }
 
-  /**
-   * DESCRIBE THE METHOD
-   *
-   * @param args DESCRIBE THE PARAMETER
-   */
   public static void main(String args[]) {
     LeafSetTest test = new LeafSetTest();
     test.start();
   }
 
-  /**
-   * DESCRIBE THE CLASS
-   *
-   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
-   * @author jeffh
-   */
-  class MyObserver implements NodeSetListener {
-    LeafSet ls;
-
-    /**
-     * Constructor for MyObserver.
-     *
-     * @param ls DESCRIBE THE PARAMETER
-     */
-    public MyObserver(LeafSet ls) {
-      this.ls = ls;
-    }
-
-    /**
-     * DESCRIBE THE METHOD
-     *
-     * @param set DESCRIBE THE PARAMETER
-     * @param handle DESCRIBE THE PARAMETER
-     * @param added DESCRIBE THE PARAMETER
-     */
-    public void nodeSetUpdate(NodeSetEventSource set, NodeHandle handle, boolean added) {
-      SimilarSet caller;
-      if (set instanceof SimilarSet) {
-        caller = (SimilarSet) set;
-      }
-
-      if (ls.overlaps() && (ls.ccwSize() != ls.cwSize())) {
-        System.out.println("FAILURE: overlaps and different size" + ls);
-      }
-      if (added) {
-        boolean consistent = !ls.directTest(handle);
-        if (!consistent) {
-          System.out.println("FAILURE:" + handle
-            + " was added, but ls is inconsistent.");
-          System.out.println(ls);
-        } else {
-          System.out.println("OK:" + handle + " was added");
-        }
-      } else {
-        // node was removed
-        boolean consistent = !ls.member(handle.getNodeId());
-        if (!consistent) {
-          System.out.println("FAILURE:" + handle
-            + " was removed, but ls is inconsistent.");
-          System.out.println(ls);
-        } else {
-          System.out.println("OK:" + handle + " was removed");
-        }
-      }
-    }
-
-  }
-
-  /**
-   * DESCRIBE THE CLASS
-   *
-   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
-   * @author jeffh
-   */
   public static class TestNodeHandle extends NodeHandle implements Comparable {
-    private NodeId id;
+    private Id id;
 
-    /**
-     * Constructor for TestNodeHandle.
-     *
-     * @param id DESCRIBE THE PARAMETER
-     */
-    public TestNodeHandle(NodeId id) {
+    public TestNodeHandle(Id id) {
       this.id = id;
     }
 
-    /**
-     * Gets the NodeId attribute of the TestNodeHandle object
-     *
-     * @return The NodeId value
-     */
-    public NodeId getNodeId() {
+    public Id getNodeId() {
       return id;
     }
 
-    /**
-     * Gets the Liveness attribute of the TestNodeHandle object
-     *
-     * @return The Liveness value
-     */
     public int getLiveness() {
       return LIVENESS_ALIVE;
     }
 
     /**
-     * DESCRIBE THE METHOD
-     *
-     * @return DESCRIBE THE RETURN VALUE
+     * @deprecated
      */
     public int proximity() {
       return 1;
     }
 
-    /**
-     * DESCRIBE THE METHOD
-     *
-     * @return DESCRIBE THE RETURN VALUE
-     */
     public boolean ping() {
       return true;
     }
 
-    /**
-     * DESCRIBE THE METHOD
-     *
-     * @param obj DESCRIBE THE PARAMETER
-     * @return DESCRIBE THE RETURN VALUE
-     */
     public boolean equals(Object obj) {
       if (obj instanceof TestNodeHandle) {
         return ((TestNodeHandle) obj).id.equals(id);
@@ -464,40 +370,26 @@ public class LeafSetTest {
       return false;
     }
 
-    /**
-     * DESCRIBE THE METHOD
-     *
-     * @return DESCRIBE THE RETURN VALUE
-     */
     public int hashCode() {
       return id.hashCode();
     }
 
     /**
-     * DESCRIBE THE METHOD
-     *
-     * @param m DESCRIBE THE PARAMETER
+     * @deprecated
      */
     public void receiveMessage(Message m) {
-    }
+    };
 
-    /**
-     * DESCRIBE THE METHOD
-     *
-     * @param o DESCRIBE THE PARAMETER
-     * @return DESCRIBE THE RETURN VALUE
-     */
     public int compareTo(Object o) {
       return id.compareTo(((TestNodeHandle) o).id);
     }
 
-    /**
-     * DESCRIBE THE METHOD
-     *
-     * @return DESCRIBE THE RETURN VALUE
-     */
     public String toString() {
       return id.toString();
     }
+
+    public void serialize(OutputBuffer buf) throws IOException {
+      throw new RuntimeException("not implemented.");        
+    }
   }
 }
