@@ -1,8 +1,9 @@
-/*************************************************************************
+/*******************************************************************************
 
-"FreePastry" Peer-to-Peer Application Development Substrate 
+"FreePastry" Peer-to-Peer Application Development Substrate
 
-Copyright 2002, Rice University. All rights reserved.
+Copyright 2002-2007, Rice University. Copyright 2006-2007, Max Planck Institute 
+for Software Systems.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -15,24 +16,24 @@ notice, this list of conditions and the following disclaimer.
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 
-- Neither  the name  of Rice  University (RICE) nor  the names  of its
-contributors may be  used to endorse or promote  products derived from
-this software without specific prior written permission.
+- Neither the name of Rice  University (RICE), Max Planck Institute for Software 
+Systems (MPI-SWS) nor the names of its contributors may be used to endorse or 
+promote products derived from this software without specific prior written 
+permission.
 
-This software is provided by RICE and the contributors on an "as is"
-basis, without any representations or warranties of any kind, express
-or implied including, but not limited to, representations or
-warranties of non-infringement, merchantability or fitness for a
-particular purpose. In no event shall RICE or contributors be liable
-for any direct, indirect, incidental, special, exemplary, or
-consequential damages (including, but not limited to, procurement of
-substitute goods or services; loss of use, data, or profits; or
-business interruption) however caused and on any theory of liability,
-whether in contract, strict liability, or tort (including negligence
-or otherwise) arising in any way out of the use of this software, even
-if advised of the possibility of such damage.
+This software is provided by RICE, MPI-SWS and the contributors on an "as is" 
+basis, without any representations or warranties of any kind, express or implied 
+including, but not limited to, representations or warranties of 
+non-infringement, merchantability or fitness for a particular purpose. In no 
+event shall RICE, MPI-SWS or contributors be liable for any direct, indirect, 
+incidental, special, exemplary, or consequential damages (including, but not 
+limited to, procurement of substitute goods or services; loss of use, data, or 
+profits; or business interruption) however caused and on any theory of 
+liability, whether in contract, strict liability, or tort (including negligence
+or otherwise) arising in any way out of the use of this software, even if 
+advised of the possibility of such damage.
 
-********************************************************************************/
+*******************************************************************************/ 
 package rice.selector;
 
 import java.io.IOException;
@@ -40,6 +41,7 @@ import java.nio.channels.*;
 import java.util.*;
 
 import rice.Destructable;
+import rice.environment.Environment;
 import rice.environment.logging.*;
 import rice.environment.time.TimeSource;
 
@@ -47,90 +49,58 @@ import rice.environment.time.TimeSource;
  * This class is the class which handles the selector, and listens for activity.
  * When activity occurs, it figures out who is interested in what has happened,
  * and hands off to that object.
- *
- * @version $Id: SelectorManager.java 3144 2006-03-15 16:19:17Z jstewart $
+ * 
+ * @version $Id: SelectorManager.java 3652 2007-03-27 17:50:45Z jeffh $
  * @author Alan Mislove
  */
 public class SelectorManager extends Thread implements Timer, Destructable {
 
+  // the maximal time to sleep on a select operation
+  public static int TIMEOUT = 500;
+
   // the underlying selector used
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected Selector selector;
 
   // a list of the invocations that need to be done in this thread
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected LinkedList invocations;
 
   // the list of handlers which want to change their key
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected HashSet modifyKeys;
 
   // the list of keys waiting to be cancelled
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected HashSet cancelledKeys;
 
   // the set used to store the timer events
-  /**
-   * DESCRIBE THE FIELD
-   */
-  protected TreeSet timerQueue = new TreeSet();
+//  protected TreeSet timerQueue = new TreeSet();
+  protected Queue timerQueue = new PriorityQueue();
 
   // the next time the selector is schedeled to wake up
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected long wakeupTime = 0;
 
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected TimeSource timeSource;
 
   long lastTime = 0;
 
 //  protected LogManager log;
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected Logger logger;
-
-  /**
-   * DESCRIBE THE FIELD
-   */
+  
   protected String instance;
 
-  /**
-   * DESCRIBE THE FIELD
-   */
   protected boolean running = true;
-
-  ArrayList loopObservers = new ArrayList();
-
-  // the maximal time to sleep on a select operation
+  
   /**
-   * DESCRIBE THE FIELD
+   * Can be disabled for the simulator to improve performance, only do this
+   * if you know you don't need to select on anything
    */
-  public static int TIMEOUT = 500;
-
+  protected boolean select = true;
+  
   /**
    * Constructor, which is private since there is only one selector per JVM.
-   *
-   * @param instance DESCRIBE THE PARAMETER
-   * @param timeSource DESCRIBE THE PARAMETER
-   * @param log DESCRIBE THE PARAMETER
    */
   public SelectorManager(String instance,
-                         TimeSource timeSource, LogManager log) {
+      TimeSource timeSource, LogManager log) {
     super(instance == null ? "Selector Thread" : "Selector Thread -- "
-      + instance);
+        + instance);
     this.instance = instance;
     this.logger = log.getLogger(getClass(), instance);
     this.invocations = new LinkedList();
@@ -142,17 +112,48 @@ public class SelectorManager extends Thread implements Timer, Destructable {
     try {
       selector = Selector.open();
     } catch (IOException e) {
-      System.out.println("SEVERE ERROR (SelectorManager): Error creating selector "
-        + e);
+      System.out
+          .println("SEVERE ERROR (SelectorManager): Error creating selector "
+              + e);
     }
     lastTime = timeSource.currentTimeMillis();
+  }
+
+  Environment environment;
+  
+  /**
+   * Can only be called once.  The rest of the time the argument is rejected.
+   * This is needed so that when the NodeFactory clones the Environment, it doesn't
+   * get restarted/have multiple different environments etc...
+   * 
+   * @param env
+   */
+  public void setEnvironment(Environment env) {
+    if (env == null) throw new IllegalArgumentException("env is null!");
+    if (environment != null) return;
+    environment = env;
     start();
   }
+  
+  /**
+   * Method which asks the Selector Manager to add the given key to the
+   * cancelled set. If noone calls register on this key during the rest of this
+   * select() operation, the key will be cancelled. Otherwise, it will be
+   * returned as a result of the register operation.
+   * 
+   * @param key The key to cancel
+   */
+  public void cancel(SelectionKey key) {
+    if (key == null)
+      throw new NullPointerException();
+
+    cancelledKeys.add(key);
+  }
 
   /**
    * Utility method which returns the SelectionKey attached to the given
    * channel, if one exists
-   *
+   * 
    * @param channel The channel to return the key for
    * @return The key
    */
@@ -160,142 +161,65 @@ public class SelectorManager extends Thread implements Timer, Destructable {
     return channel.keyFor(selector);
   }
 
+  /**
+   * Registers a new channel with the selector, and attaches the given
+   * SelectionKeyHandler as the handler for the newly created key. Operations
+   * which the hanlder is interested in will be called as available.
+   * 
+   * @param channel The channel to regster with the selector
+   * @param handler The handler to use for the callbacks
+   * @param ops The initial interest operations
+   * @return The SelectionKey which uniquely identifies this channel
+   */
+  public SelectionKey register(SelectableChannel channel,
+      SelectionKeyHandler handler, int ops) throws IOException {
+    if ((channel == null) || (handler == null))
+      throw new NullPointerException();
+
+    SelectionKey key = channel.register(selector, ops, handler);
+    if (cancelledKeys != null) cancelledKeys.remove(key);
+
+    return key;
+  }
+  
+  /**
+   * This method schedules a runnable task to be done by the selector thread
+   * during the next select() call. All operations which modify the selector
+   * should be done using this method, as they must be done in the selector
+   * thread.
+   * 
+   * @param d The runnable task to invoke
+   */
+  public synchronized void invoke(Runnable d) {
+    if (d == null)
+      throw new NullPointerException();
+
+    invocations.add(d);
+    wakeup();
+  }
+
   /**
    * Debug method which returns the number of pending invocations
-   *
+   * 
    * @return The number of pending invocations
    */
   public int getNumInvocations() {
     return invocations.size();
   }
 
-  /**
-   * Method which synchroniously returns the first element off of the
-   * invocations list.
-   *
-   * @return An item from the invocations list
-   */
-  protected synchronized Runnable getInvocation() {
-    if (invocations.size() > 0) {
-      return (Runnable) invocations.removeFirst();
-    } else {
-      return null;
-    }
-  }
-
-  /**
-   * Method which synchroniously returns on element off of the modifyKeys list
-   *
-   * @return An item from the invocations list
-   */
-  protected synchronized SelectionKey getModifyKey() {
-    if (modifyKeys.size() > 0) {
-      Object result = modifyKeys.iterator().next();
-      modifyKeys.remove(result);
-      return (SelectionKey) result;
-    } else {
-      return null;
-    }
-  }
-
-  /**
-   * Returns whether or not this thread of execution is the selector thread
-   *
-   * @return Whether or not this is the selector thread
-   */
-  public boolean isSelectorThread() {
-    return (Thread.currentThread() == this);
-  }
-
-  /**
-   * Returns the timer associated with this SelectorManager (in this case, it is
-   * this).
-   *
-   * @return The associated timer
-   */
-  public Timer getTimer() {
-    return this;
-  }
-
-  /**
-   * Gets the Selector attribute of the SelectorManager object
-   *
-   * @return The Selector value
-   */
-  public Selector getSelector() {
-    return selector;
-  }
-
-  /**
-   * Method which asks the Selector Manager to add the given key to the
-   * cancelled set. If noone calls register on this key during the rest of this
-   * select() operation, the key will be cancelled. Otherwise, it will be
-   * returned as a result of the register operation.
-   *
-   * @param key The key to cancel
-   */
-  public void cancel(SelectionKey key) {
-    if (key == null) {
-      throw new NullPointerException();
-    }
-
-    cancelledKeys.add(key);
-  }
-
-  /**
-   * Registers a new channel with the selector, and attaches the given
-   * SelectionKeyHandler as the handler for the newly created key. Operations
-   * which the hanlder is interested in will be called as available.
-   *
-   * @param channel The channel to regster with the selector
-   * @param handler The handler to use for the callbacks
-   * @param ops The initial interest operations
-   * @return The SelectionKey which uniquely identifies this channel
-   * @exception IOException DESCRIBE THE EXCEPTION
-   */
-  public SelectionKey register(SelectableChannel channel,
-                               SelectionKeyHandler handler, int ops) throws IOException {
-    if ((channel == null) || (handler == null)) {
-      throw new NullPointerException();
-    }
-
-    SelectionKey key = channel.register(selector, ops, handler);
-    cancelledKeys.remove(key);
-
-    return key;
-  }
-
-  /**
-   * This method schedules a runnable task to be done by the selector thread
-   * during the next select() call. All operations which modify the selector
-   * should be done using this method, as they must be done in the selector
-   * thread.
-   *
-   * @param d The runnable task to invoke
-   */
-  public synchronized void invoke(Runnable d) {
-    if (d == null) {
-      throw new NullPointerException();
-    }
-
-    invocations.add(d);
-    selector.wakeup();
-  }
-
   /**
    * Adds a selectionkey handler into the list of handlers which wish to change
    * their keys. Thus, modifyKeys() will be called on the next selection
    * operation
-   *
+   * 
    * @param key The key which is to be chanegd
    */
   public synchronized void modifyKey(SelectionKey key) {
-    if (key == null) {
+    if (key == null)
       throw new NullPointerException();
-    }
 
     modifyKeys.add(key);
-    selector.wakeup();
+    wakeup();
   }
 
   /**
@@ -309,16 +233,15 @@ public class SelectorManager extends Thread implements Timer, Destructable {
    * to be started when this thread's start() method is invoked.
    */
   public void run() {
-    try {
-      //System.out.println("SelectorManager starting...");
-      if (logger.level <= Logger.INFO) {
-        logger.log("SelectorManager -- " + instance + " starting...");
-      }
+    //System.out.println("SelectorManager starting...");
+    if (logger.level <= Logger.INFO) logger.log("SelectorManager -- " + instance + " starting...");
 
-      lastTime = timeSource.currentTimeMillis();
-      // loop while waiting for activity
-      while (running) {
-        notifyLoopListeners();
+    lastTime = timeSource.currentTimeMillis();
+    // loop while waiting for activity
+    while (running) {
+      try {
+        if (useLoopListeners)
+          notifyLoopListeners();
 
         // NOTE: This is so we aren't always holding the selector lock when we
         // get context switched
@@ -326,22 +249,22 @@ public class SelectorManager extends Thread implements Timer, Destructable {
         executeDueTasks();
         onLoop();
         doInvocations();
-        doSelections();
-        synchronized (selector) {
+        if (select) {
+          doSelections();
           int selectTime = SelectorManager.TIMEOUT;
           if (timerQueue.size() > 0) {
-            TimerTask first = (TimerTask) timerQueue.first();
-            selectTime = (int) (first.nextExecutionTime - timeSource.currentTimeMillis());
+            TimerTask first = (TimerTask) timerQueue.peek();
+            selectTime = (int) (first.nextExecutionTime - timeSource
+                .currentTimeMillis());
           }
 
           select(selectTime);
-
+          
           if (cancelledKeys.size() > 0) {
             Iterator i = cancelledKeys.iterator();
 
-            while (i.hasNext()) {
+            while (i.hasNext())
               ((SelectionKey) i.next()).cancel();
-            }
 
             cancelledKeys.clear();
 
@@ -349,31 +272,45 @@ public class SelectorManager extends Thread implements Timer, Destructable {
             // cancelled (dumb)
             selector.selectNow();
           }
-        }
+        } // if select
+      } catch (Throwable t) {
+        if (logger.level <= Logger.SEVERE) logger.logException(
+            "ERROR (SelectorManager.run): " , t);
+        environment.getExceptionStrategy().handleException(this, t);
+        System.exit(-1);
       }
-    } catch (Throwable t) {
-      if (logger.level <= Logger.SEVERE) {
-        logger.logException(
-          "ERROR (SelectorManager.run): ", t);
-      }
-      System.exit(-1);
-    }
-    if (logger.level <= Logger.INFO) {
-      logger.log("Selector " + instance + " shutting down.");
     }
+    invocations.clear();
+    loopObservers.clear();
+    cancelledKeys.clear();
+    timerQueue.clear();
+    invocations = null;
+    loopObservers = null;
+    cancelledKeys = null;
+    timerQueue = null;
+    if (logger.level <= Logger.INFO) logger.log("Selector "+instance+" shutting down.");
   }
-
-
-  /**
-   * DESCRIBE THE METHOD
-   */
+  
+  @SuppressWarnings("deprecation")
   public void destroy() {
-    running = false;
+    running = false; 
   }
-
+  
   /**
-   * DESCRIBE THE METHOD
+   * LoopListeners is used in case you are worried that your process may not get scheduled for a while
+   * such as on an overloaded planetlab node, or if you hibernate your laptop, this is not needed 
+   * for the simulator
    */
+  protected boolean useLoopListeners = true;
+  
+  /**
+   * Set this to false when using the simulator, because you don't need to notify loop observers.
+   * @param val
+   */
+  public void useLoopListeners(boolean val) {
+    useLoopListeners = val;
+  }
+  
   protected void notifyLoopListeners() {
     long now = timeSource.currentTimeMillis();
     long diff = now - lastTime;
@@ -382,7 +319,7 @@ public class SelectorManager extends Thread implements Timer, Destructable {
       Iterator i = loopObservers.iterator();
       while (i.hasNext()) {
         LoopObserver lo = (LoopObserver) i.next();
-        if (lo.delayInterest() >= diff) {
+        if (lo.delayInterest() <= diff) {
           lo.loopTime((int) diff);
         }
       }
@@ -390,36 +327,23 @@ public class SelectorManager extends Thread implements Timer, Destructable {
     lastTime = now;
   }
 
-  /**
-   * Adds a feature to the LoopObserver attribute of the SelectorManager object
-   *
-   * @param lo The feature to be added to the LoopObserver attribute
-   */
+  ArrayList loopObservers = new ArrayList();
+
   public void addLoopObserver(LoopObserver lo) {
     synchronized (loopObservers) {
       loopObservers.add(lo);
     }
   }
 
-  /**
-   * DESCRIBE THE METHOD
-   *
-   * @param lo DESCRIBE THE PARAMETER
-   */
   public void removeLoopObserver(LoopObserver lo) {
     synchronized (loopObservers) {
       loopObservers.remove(lo);
     }
   }
 
-  /**
-   * DESCRIBE THE METHOD
-   *
-   * @exception IOException DESCRIBE THE EXCEPTION
-   */
   protected void doSelections() throws IOException {
     SelectionKey[] keys = selectedKeys();
-
+    
     // to debug weird selection bug
     if (keys.length > 1000 && logger.level <= Logger.FINE) {
       logger.log("lots of selection keys!");
@@ -427,7 +351,7 @@ public class SelectorManager extends Thread implements Timer, Destructable {
       for (int i = 0; i < keys.length; i++) {
         String keyclass = keys[i].getClass().getName();
         if (histo.containsKey(keyclass)) {
-          histo.put(keyclass, new Integer(((Integer) histo.get(keyclass)).intValue() + 1));
+          histo.put(keyclass, new Integer(((Integer)histo.get(keyclass)).intValue() + 1));
         } else {
           histo.put(keyclass, new Integer(1));
         }
@@ -435,8 +359,8 @@ public class SelectorManager extends Thread implements Timer, Destructable {
       logger.log("begin selection keys by class");
       Iterator it = histo.keySet().iterator();
       while (it.hasNext()) {
-        String name = (String) it.next();
-        logger.log("Selection Key: " + name + ": " + histo.get(name));
+        String name = (String)it.next();
+        logger.log("Selection Key: " + name + ": "+histo.get(name));
       }
       logger.log("end selection keys by class");
     }
@@ -491,10 +415,8 @@ public class SelectorManager extends Thread implements Timer, Destructable {
       try {
         run.run();
       } catch (Exception e) {
-        if (logger.level <= Logger.SEVERE) {
-          logger.logException(
-            "Invoking runnable caused exception " + e + " - continuing", e);
-        }
+        if (logger.level <= Logger.SEVERE) logger.logException( 
+            "Invoking runnable caused exception " + e + " - continuing",e);
       }
     }
 
@@ -505,42 +427,62 @@ public class SelectorManager extends Thread implements Timer, Destructable {
 
     while (i.hasNext()) {
       SelectionKey key = (SelectionKey) i.next();
-      if (key.isValid() && (key.attachment() != null)) {
+      if (key.isValid() && (key.attachment() != null))
         ((SelectionKeyHandler) key.attachment()).modifyKey(key);
-      }
+    }
+  }
+
+  /**
+   * Method which synchroniously returns the first element off of the
+   * invocations list.
+   * 
+   * @return An item from the invocations list
+   */
+  protected synchronized Runnable getInvocation() {
+    if (invocations.size() > 0)
+      return (Runnable) invocations.removeFirst();
+    else
+      return null;
+  }
+
+  /**
+   * Method which synchroniously returns on element off of the modifyKeys list
+   * 
+   * @return An item from the invocations list
+   */
+  protected synchronized SelectionKey getModifyKey() {
+    if (modifyKeys.size() > 0) {
+      Object result = modifyKeys.iterator().next();
+      modifyKeys.remove(result);
+      return (SelectionKey) result;
+    } else {
+      return null;
     }
   }
 
   /**
    * Selects on the selector, and returns the result. Also properly synchronizes
    * around the selector
-   *
-   * @param time DESCRIBE THE PARAMETER
+   * 
    * @return DESCRIBE THE RETURN VALUE
    * @exception IOException DESCRIBE THE EXCEPTION
    */
   int select(int time) throws IOException {
-    if (time > TIMEOUT) {
+    if (time > TIMEOUT)
       time = TIMEOUT;
-    }
 
     try {
-      if ((time <= 0) || (invocations.size() > 0) || (modifyKeys.size() > 0)) {
+      if ((time <= 0) || (invocations.size() > 0) || (modifyKeys.size() > 0))
         return selector.selectNow();
-      }
 
       wakeupTime = timeSource.currentTimeMillis() + time;
       return selector.select(time);
     } catch (CancelledKeyException cce) {
-      if (logger.level <= Logger.WARNING) {
-        logger.logException("CCE: cause:", cce.getCause());
-      }
+      if (logger.level <= Logger.WARNING) logger.logException("CCE: cause:",cce.getCause());
       throw cce;
     } catch (IOException e) {
       if (e.getMessage().indexOf("Interrupted system call") >= 0) {
-        if (logger.level <= Logger.WARNING) {
-          logger.log("Got interrupted system call, continuing anyway...");
-        }
+        if (logger.level <= Logger.WARNING) logger.log("Got interrupted system call, continuing anyway...");
         return 1;
       } else {
         throw e;
@@ -551,7 +493,7 @@ public class SelectorManager extends Thread implements Timer, Destructable {
   /**
    * Selects all of the keys on the selector and returns the result as an array
    * of keys.
-   *
+   * 
    * @return The array of keys
    * @exception IOException DESCRIBE THE EXCEPTION
    */
@@ -562,21 +504,33 @@ public class SelectorManager extends Thread implements Timer, Destructable {
   /**
    * Selects all of the currenlty selected keys on the selector and returns the
    * result as an array of keys.
-   *
+   * 
    * @return The array of keys
    * @exception IOException DESCRIBE THE EXCEPTION
    */
   protected SelectionKey[] selectedKeys() throws IOException {
     return (SelectionKey[]) selector.selectedKeys()
-      .toArray(new SelectionKey[0]);
+        .toArray(new SelectionKey[0]);
   }
 
   /**
-   * Method which schedules a task to run after a specified number of millis
-   *
-   * @param task The task to run
-   * @param delay The delay before running, in milliseconds
+   * Returns whether or not this thread of execution is the selector thread
+   * 
+   * @return Whether or not this is the selector thread
    */
+  public boolean isSelectorThread() {
+    return (Thread.currentThread() == this);
+  }
+
+  /**
+   * Method which schedules a task to run after a specified number of millis.  The task must have a proper nextExecutionTime set
+   * 
+   * @param task The task to run
+   */
+  public void schedule(TimerTask task) {
+    addTask(task);
+  }
+  
   public void schedule(TimerTask task, long delay) {
     task.nextExecutionTime = timeSource.currentTimeMillis() + delay;
     addTask(task);
@@ -585,7 +539,7 @@ public class SelectorManager extends Thread implements Timer, Destructable {
   /**
    * Method which schedules a task to run repeatedly after a specified delay and
    * period
-   *
+   * 
    * @param task The task to run
    * @param delay The delay before first running, in milliseconds
    * @param period The period with which to run in milliseconds
@@ -599,7 +553,7 @@ public class SelectorManager extends Thread implements Timer, Destructable {
   /**
    * Method which schedules a task to run repeatedly (at a fixed rate) after a
    * specified delay and period
-   *
+   * 
    * @param task The task to run
    * @param delay The delay before first running in milliseconds
    * @param period The period with which to run in milliseconds
@@ -614,24 +568,61 @@ public class SelectorManager extends Thread implements Timer, Destructable {
   /**
    * Internal method which adds a task to the task tree, waking up the selector
    * if necessary to recalculate the sleep time
-   *
+   * 
    * @param task The task to add
    */
-  private void addTask(TimerTask task) {
-    synchronized (selector) {
+  private synchronized void addTask(TimerTask task) {
+//    synchronized (selector) {
       if (!timerQueue.add(task)) {
         System.out.println("ERROR: Got false while enqueueing task " + task
-          + "!");
+            + "!");
         Thread.dumpStack();
+      } else {
+        task.setSelectorManager(this); 
       }
-    }
 
     // need to interrupt thread if waiting too long in selector
-    if (wakeupTime >= task.scheduledExecutionTime()) {
-      selector.wakeup();
+    if (select) {
+      // using the network
+      if (wakeupTime >= task.scheduledExecutionTime()) {
+        // we need to wake up the selector because it's going to sleep too long
+        wakeup();
+      }
+    } else {
+      // using the simulator
+      if (task.scheduledExecutionTime() == getNextTaskExecutionTime()) {
+        // we need to wake up the selector because we are now the newest 
+        // shortest wait, and may be delaying because of a later event
+        wakeup();
+      }
     }
+    
+//    } // synchronized
+
+  }
+  
+  public synchronized void removeTask(TimerTask task) {
+    timerQueue.remove(task);
   }
 
+  /**
+   * Note, should hold the selector's (this) lock to call this.
+   *
+   */
+  public void wakeup() {
+    selector.wakeup();
+    this.notifyAll();
+  }
+
+  public long getNextTaskExecutionTime() {
+//    if (!invocations.isEmpty()) return timeSource.currentTimeMillis();
+    if (timerQueue.size() > 0) {
+      TimerTask next = (TimerTask) timerQueue.peek();
+      return next.nextExecutionTime;
+    }
+    return -1;    
+  }
+  
   /**
    * Internal method which finds all due tasks and executes them.
    */
@@ -641,15 +632,16 @@ public class SelectorManager extends Thread implements Timer, Destructable {
     ArrayList executeNow = new ArrayList();
 
     // step 1, fetch all due timers
-    synchronized (selector) {
+    synchronized (this) {
       boolean done = false;
       while (!done) {
         if (timerQueue.size() > 0) {
-          TimerTask next = (TimerTask) timerQueue.first();
+          TimerTask next = (TimerTask) timerQueue.peek();
           if (next.nextExecutionTime <= now) {
             executeNow.add(next);
             //System.out.println("Removing:"+next);
-            timerQueue.remove(next);
+//            timerQueue.remove(next);
+            timerQueue.poll();
           } else {
             done = true;
           }
@@ -671,14 +663,12 @@ public class SelectorManager extends Thread implements Timer, Destructable {
           addBack.add(next);
         }
       } catch (Exception e) {
-        if (logger.level <= Logger.SEVERE) {
-          logger.logException("", e);
-        }
+        if (logger.level <= Logger.SEVERE) logger.logException("",e);
       }
     }
 
     // step 3, add them back if necessary
-    synchronized (selector) {
+    synchronized (this) {
       i = addBack.iterator();
       while (i.hasNext()) {
         TimerTask tt = (TimerTask) i.next();
@@ -687,4 +677,23 @@ public class SelectorManager extends Thread implements Timer, Destructable {
       }
     }
   }
+
+  /**
+   * Returns the timer associated with this SelectorManager (in this case, it is
+   * this).
+   * 
+   * @return The associated timer
+   */
+  public Timer getTimer() {
+    return this;
+  }
+  
+  public Selector getSelector() {
+    return selector; 
+  }
+
+  public void setSelect(boolean b) {
+    select = b;
+  }
+
 }
